var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });

// dist/duration-format.js
var unitNames = ["second", "minute", "hour", "day", "month", "year"];
function timeAgo(date) {
  const ms = new Date().getTime() - date.getTime();
  const sec = Math.round(ms / 1e3);
  const min = Math.round(sec / 60);
  const hr = Math.round(min / 60);
  const day = Math.round(hr / 24);
  const month = Math.round(day / 30);
  const year = Math.round(month / 12);
  if (ms < 0) {
    return [0, "second"];
  } else if (sec < 10) {
    return [0, "second"];
  } else if (sec < 45) {
    return [-sec, "second"];
  } else if (sec < 90) {
    return [-min, "minute"];
  } else if (min < 45) {
    return [-min, "minute"];
  } else if (min < 90) {
    return [-hr, "hour"];
  } else if (hr < 24) {
    return [-hr, "hour"];
  } else if (hr < 36) {
    return [-day, "day"];
  } else if (day < 30) {
    return [-day, "day"];
  } else if (month < 18) {
    return [-month, "month"];
  } else {
    return [-year, "year"];
  }
}
__name(timeAgo, "timeAgo");
function microTimeAgo(date) {
  const ms = new Date().getTime() - date.getTime();
  const sec = Math.round(ms / 1e3);
  const min = Math.round(sec / 60);
  const hr = Math.round(min / 60);
  const day = Math.round(hr / 24);
  const month = Math.round(day / 30);
  const year = Math.round(month / 12);
  if (min < 1) {
    return [1, "minute"];
  } else if (min < 60) {
    return [min, "minute"];
  } else if (hr < 24) {
    return [hr, "hour"];
  } else if (day < 365) {
    return [day, "day"];
  } else {
    return [year, "year"];
  }
}
__name(microTimeAgo, "microTimeAgo");
function timeUntil(date) {
  const ms = date.getTime() - new Date().getTime();
  const sec = Math.round(ms / 1e3);
  const min = Math.round(sec / 60);
  const hr = Math.round(min / 60);
  const day = Math.round(hr / 24);
  const month = Math.round(day / 30);
  const year = Math.round(month / 12);
  if (month >= 18) {
    return [year, "year"];
  } else if (month >= 12) {
    return [year, "year"];
  } else if (day >= 45) {
    return [month, "month"];
  } else if (day >= 30) {
    return [month, "month"];
  } else if (hr >= 36) {
    return [day, "day"];
  } else if (hr >= 24) {
    return [day, "day"];
  } else if (min >= 90) {
    return [hr, "hour"];
  } else if (min >= 45) {
    return [hr, "hour"];
  } else if (sec >= 90) {
    return [min, "minute"];
  } else if (sec >= 45) {
    return [min, "minute"];
  } else if (sec >= 10) {
    return [sec, "second"];
  } else {
    return [0, "second"];
  }
}
__name(timeUntil, "timeUntil");
function microTimeUntil(date) {
  const ms = date.getTime() - new Date().getTime();
  const sec = Math.round(ms / 1e3);
  const min = Math.round(sec / 60);
  const hr = Math.round(min / 60);
  const day = Math.round(hr / 24);
  const month = Math.round(day / 30);
  const year = Math.round(month / 12);
  if (day >= 365) {
    return [year, "year"];
  } else if (hr >= 24) {
    return [day, "day"];
  } else if (min >= 60) {
    return [hr, "hour"];
  } else if (min > 1) {
    return [min, "minute"];
  } else {
    return [1, "minute"];
  }
}
__name(microTimeUntil, "microTimeUntil");
function elapsedTime(date) {
  const ms = Math.abs(date.getTime() - new Date().getTime());
  const sec = Math.floor(ms / 1e3);
  const min = Math.floor(sec / 60);
  const hr = Math.floor(min / 60);
  const day = Math.floor(hr / 24);
  const month = Math.floor(day / 30);
  const year = Math.floor(month / 12);
  const units = [];
  if (year)
    units.push([year, "year"]);
  if (month - year * 12)
    units.push([month - year * 12, "month"]);
  if (day - month * 30)
    units.push([day - month * 30, "day"]);
  if (hr - day * 24)
    units.push([hr - day * 24, "hour"]);
  if (min - hr * 60)
    units.push([min - hr * 60, "minute"]);
  if (sec - min * 60)
    units.push([sec - min * 60, "second"]);
  return units;
}
__name(elapsedTime, "elapsedTime");

// dist/duration.js
var durationRe = /^[-+]?P(?:(\d+)Y)?(?:(\d+)M)?(?:(\d+)W)?(?:(\d+)D)?(?:T(?:(\d+)H)?(?:(\d+)M)?(?:(\d+)S)?)?$/;
var isDuration = /* @__PURE__ */ __name((str) => durationRe.test(str), "isDuration");
var Duration = class {
  constructor(years = 0, months = 0, weeks = 0, days = 0, hours = 0, minutes = 0, seconds = 0) {
    this.years = years;
    this.months = months;
    this.weeks = weeks;
    this.days = days;
    this.hours = hours;
    this.minutes = minutes;
    this.seconds = seconds;
  }
  abs() {
    return new Duration(Math.abs(this.years), Math.abs(this.months), Math.abs(this.weeks), Math.abs(this.days), Math.abs(this.hours), Math.abs(this.minutes), Math.abs(this.seconds));
  }
  static from(durationLike) {
    var _a;
    if (typeof durationLike === "string") {
      const str = String(durationLike).trim();
      const factor = str.startsWith("-") ? -1 : 1;
      const parsed = (_a = str.match(durationRe)) === null || _a === void 0 ? void 0 : _a.slice(1).map((x) => (Number(x) || 0) * factor);
      if (!parsed)
        return new Duration();
      return new Duration(...parsed);
    } else if (typeof durationLike === "object") {
      const { years, months, weeks, days, hours, minutes, seconds } = durationLike;
      return new Duration(years, months, weeks, days, hours, minutes, seconds);
    }
    throw new RangeError("invalid duration");
  }
};
__name(Duration, "Duration");
function applyDuration(date, duration) {
  const r = new Date(date);
  r.setFullYear(r.getFullYear() + duration.years);
  r.setMonth(r.getMonth() + duration.months);
  r.setDate(r.getDate() + duration.weeks * 7 + duration.days);
  r.setHours(r.getHours() + duration.hours);
  r.setMinutes(r.getMinutes() + duration.minutes);
  r.setSeconds(r.getSeconds() + duration.seconds);
  return r;
}
__name(applyDuration, "applyDuration");
function withinDuration(a, b, str) {
  const duration = Duration.from(str).abs();
  const threshold = applyDuration(a, duration);
  if (!threshold)
    return true;
  return Math.abs(Number(threshold) - Number(a)) > Math.abs(Number(a) - Number(b));
}
__name(withinDuration, "withinDuration");

// dist/relative-time-element.js
var __classPrivateFieldGet = function(receiver, state, kind, f) {
  if (kind === "a" && !f)
    throw new TypeError("Private accessor was defined without a getter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
    throw new TypeError("Cannot read private member from an object whose class did not declare it");
  return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = function(receiver, state, value, kind, f) {
  if (kind === "m")
    throw new TypeError("Private method is not writable");
  if (kind === "a" && !f)
    throw new TypeError("Private accessor was defined without a setter");
  if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
    throw new TypeError("Cannot write private member to an object whose class did not declare it");
  return kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value), value;
};
var _RelativeTimeElement_instances;
var _RelativeTimeElement_customTitle;
var _RelativeTimeElement_updating;
var _RelativeTimeElement_lang_get;
var _RelativeTimeElement_renderRoot;
var _RelativeTimeElement_getFormattedTitle;
var _RelativeTimeElement_getFormattedDate;
var root = typeof globalThis !== "undefined" ? globalThis : window;
var HTMLElement = root.HTMLElement || null;
var RelativeTimeUpdatedEvent = class extends Event {
  constructor(oldText, newText, oldTitle, newTitle) {
    super("relative-time-updated", { bubbles: true, composed: true });
    this.oldText = oldText;
    this.newText = newText;
    this.oldTitle = oldTitle;
    this.newTitle = newTitle;
  }
};
__name(RelativeTimeUpdatedEvent, "RelativeTimeUpdatedEvent");
function getUnitFactor(el) {
  if (!el.date)
    return Infinity;
  if (el.format === "elapsed") {
    const precision = el.precision;
    if (precision === "second") {
      return 1e3;
    } else if (precision === "minute") {
      return 60 * 1e3;
    }
  }
  const ms = Math.abs(Date.now() - el.date.getTime());
  if (ms < 60 * 1e3)
    return 1e3;
  if (ms < 60 * 60 * 1e3)
    return 60 * 1e3;
  return 60 * 60 * 1e3;
}
__name(getUnitFactor, "getUnitFactor");
var dateObserver = new class {
  constructor() {
    this.elements = /* @__PURE__ */ new Set();
    this.time = Infinity;
    this.timer = -1;
  }
  observe(element) {
    if (this.elements.has(element))
      return;
    this.elements.add(element);
    const date = element.date;
    if (date && date.getTime()) {
      const ms = getUnitFactor(element);
      const time = Date.now() + ms;
      if (time < this.time) {
        clearTimeout(this.timer);
        this.timer = setTimeout(() => this.update(), ms);
        this.time = time;
      }
    }
  }
  unobserve(element) {
    if (!this.elements.has(element))
      return;
    this.elements.delete(element);
  }
  update() {
    clearTimeout(this.timer);
    if (!this.elements.size)
      return;
    let nearestDistance = Infinity;
    for (const timeEl of this.elements) {
      nearestDistance = Math.min(nearestDistance, getUnitFactor(timeEl));
      timeEl.update();
    }
    this.time = Math.min(60 * 60 * 1e3, nearestDistance);
    this.timer = setTimeout(() => this.update(), this.time);
    this.time += Date.now();
  }
}();
var RelativeTimeElement = class extends HTMLElement {
  constructor() {
    super(...arguments);
    _RelativeTimeElement_instances.add(this);
    _RelativeTimeElement_customTitle.set(this, false);
    _RelativeTimeElement_updating.set(this, false);
    _RelativeTimeElement_renderRoot.set(this, this.shadowRoot ? this.shadowRoot : this.attachShadow ? this.attachShadow({ mode: "open" }) : this);
  }
  static get observedAttributes() {
    return [
      "second",
      "minute",
      "hour",
      "weekday",
      "day",
      "month",
      "year",
      "time-zone-name",
      "prefix",
      "threshold",
      "tense",
      "precision",
      "format",
      "datetime",
      "lang",
      "title"
    ];
  }
  get second() {
    const second = this.getAttribute("second");
    if (second === "numeric" || second === "2-digit")
      return second;
  }
  set second(value) {
    this.setAttribute("second", value || "");
  }
  get minute() {
    const minute = this.getAttribute("minute");
    if (minute === "numeric" || minute === "2-digit")
      return minute;
  }
  set minute(value) {
    this.setAttribute("minute", value || "");
  }
  get hour() {
    const hour = this.getAttribute("hour");
    if (hour === "numeric" || hour === "2-digit")
      return hour;
  }
  set hour(value) {
    this.setAttribute("hour", value || "");
  }
  get weekday() {
    const weekday = this.getAttribute("weekday");
    if (weekday === "long" || weekday === "short" || weekday === "narrow")
      return weekday;
  }
  set weekday(value) {
    this.setAttribute("weekday", value || "");
  }
  get day() {
    var _a;
    const day = (_a = this.getAttribute("day")) !== null && _a !== void 0 ? _a : "numeric";
    if (day === "numeric" || day === "2-digit")
      return day;
  }
  set day(value) {
    this.setAttribute("day", value || "");
  }
  get month() {
    var _a;
    const month = (_a = this.getAttribute("month")) !== null && _a !== void 0 ? _a : "short";
    if (month === "numeric" || month === "2-digit" || month === "short" || month === "long" || month === "narrow") {
      return month;
    }
  }
  set month(value) {
    this.setAttribute("month", value || "");
  }
  get year() {
    var _a;
    const year = this.getAttribute("year");
    if (year === "numeric" || year === "2-digit")
      return year;
    if (!this.hasAttribute("year") && new Date().getUTCFullYear() !== ((_a = this.date) === null || _a === void 0 ? void 0 : _a.getUTCFullYear())) {
      return "numeric";
    }
  }
  set year(value) {
    this.setAttribute("day", value || "");
  }
  get timeZoneName() {
    const name = this.getAttribute("time-zone-name");
    if (name === "long" || name === "short" || name === "shortOffset" || name === "longOffset" || name === "shortGeneric" || name === "longGeneric") {
      return name;
    }
  }
  set timeZoneName(value) {
    this.setAttribute("time-zone-name", value || "");
  }
  get prefix() {
    var _a;
    return (_a = this.getAttribute("prefix")) !== null && _a !== void 0 ? _a : "on";
  }
  set prefix(value) {
    this.setAttribute("prefix", value);
  }
  get threshold() {
    const threshold = this.getAttribute("threshold");
    return threshold && isDuration(threshold) ? threshold : "P30D";
  }
  set threshold(value) {
    this.setAttribute("threshold", value);
  }
  get tense() {
    const tense = this.getAttribute("tense");
    if (tense === "past")
      return "past";
    if (tense === "future")
      return "future";
    return "auto";
  }
  set tense(value) {
    this.setAttribute("tense", value);
  }
  get precision() {
    const precision = this.getAttribute("precision");
    if (unitNames.includes(precision))
      return precision;
    return "second";
  }
  set precision(value) {
    this.setAttribute("precision", value);
  }
  get format() {
    const format = this.getAttribute("format");
    if (format === "micro")
      return "micro";
    if (format === "elapsed")
      return "elapsed";
    return "auto";
  }
  set format(value) {
    this.setAttribute("format", value);
  }
  get datetime() {
    return this.getAttribute("datetime") || "";
  }
  set datetime(value) {
    this.setAttribute("datetime", value);
  }
  get date() {
    const parsed = Date.parse(this.datetime);
    return Number.isNaN(parsed) ? null : new Date(parsed);
  }
  set date(value) {
    this.datetime = (value === null || value === void 0 ? void 0 : value.toISOString()) || "";
  }
  connectedCallback() {
    this.update();
  }
  disconnectedCallback() {
    dateObserver.unobserve(this);
  }
  attributeChangedCallback(attrName, oldValue, newValue) {
    if (oldValue === newValue)
      return;
    if (attrName === "title") {
      __classPrivateFieldSet(this, _RelativeTimeElement_customTitle, newValue !== null && __classPrivateFieldGet(this, _RelativeTimeElement_instances, "m", _RelativeTimeElement_getFormattedTitle).call(this) !== newValue, "f");
    }
    if (!__classPrivateFieldGet(this, _RelativeTimeElement_updating, "f") && !(attrName === "title" && __classPrivateFieldGet(this, _RelativeTimeElement_customTitle, "f"))) {
      __classPrivateFieldSet(this, _RelativeTimeElement_updating, (async () => {
        await Promise.resolve();
        this.update();
      })(), "f");
    }
  }
  update() {
    const oldText = __classPrivateFieldGet(this, _RelativeTimeElement_renderRoot, "f").textContent || "";
    const oldTitle = this.getAttribute("title") || "";
    let newTitle = oldTitle;
    let newText = oldText;
    const now = new Date();
    if (!__classPrivateFieldGet(this, _RelativeTimeElement_customTitle, "f")) {
      newTitle = __classPrivateFieldGet(this, _RelativeTimeElement_instances, "m", _RelativeTimeElement_getFormattedTitle).call(this) || "";
      if (newTitle)
        this.setAttribute("title", newTitle);
    }
    newText = __classPrivateFieldGet(this, _RelativeTimeElement_instances, "m", _RelativeTimeElement_getFormattedDate).call(this, now) || "";
    if (newText) {
      __classPrivateFieldGet(this, _RelativeTimeElement_renderRoot, "f").textContent = newText;
    } else if (this.shadowRoot === __classPrivateFieldGet(this, _RelativeTimeElement_renderRoot, "f") && this.textContent) {
      __classPrivateFieldGet(this, _RelativeTimeElement_renderRoot, "f").textContent = this.textContent;
    }
    if (newText !== oldText || newTitle !== oldTitle) {
      this.dispatchEvent(new RelativeTimeUpdatedEvent(oldText, newText, oldTitle, newTitle));
    }
    const date = this.date;
    const format = this.format;
    const isRelative = (format === "auto" || format === "micro") && date && withinDuration(now, date, this.threshold);
    if (format === "elapsed" || isRelative) {
      dateObserver.observe(this);
    } else {
      dateObserver.unobserve(this);
    }
    __classPrivateFieldSet(this, _RelativeTimeElement_updating, false, "f");
  }
};
__name(RelativeTimeElement, "RelativeTimeElement");
_RelativeTimeElement_customTitle = /* @__PURE__ */ new WeakMap(), _RelativeTimeElement_updating = /* @__PURE__ */ new WeakMap(), _RelativeTimeElement_renderRoot = /* @__PURE__ */ new WeakMap(), _RelativeTimeElement_instances = /* @__PURE__ */ new WeakSet(), _RelativeTimeElement_lang_get = /* @__PURE__ */ __name(function _RelativeTimeElement_lang_get2() {
  var _a, _b;
  return (_b = (_a = this.closest("[lang]")) === null || _a === void 0 ? void 0 : _a.getAttribute("lang")) !== null && _b !== void 0 ? _b : "default";
}, "_RelativeTimeElement_lang_get"), _RelativeTimeElement_getFormattedTitle = /* @__PURE__ */ __name(function _RelativeTimeElement_getFormattedTitle2() {
  const date = this.date;
  if (!date)
    return;
  if (typeof Intl === "undefined" || !Intl.DateTimeFormat)
    return;
  return new Intl.DateTimeFormat(__classPrivateFieldGet(this, _RelativeTimeElement_instances, "a", _RelativeTimeElement_lang_get), {
    day: "numeric",
    month: "short",
    year: "numeric",
    hour: "numeric",
    minute: "2-digit",
    timeZoneName: "short"
  }).format(date);
}, "_RelativeTimeElement_getFormattedTitle"), _RelativeTimeElement_getFormattedDate = /* @__PURE__ */ __name(function _RelativeTimeElement_getFormattedDate2(now = new Date()) {
  const date = this.date;
  if (!date)
    return;
  const format = this.format;
  if (format === "elapsed") {
    const precisionIndex = unitNames.indexOf(this.precision) || 0;
    const units = elapsedTime(date).filter((unit) => unitNames.indexOf(unit[1]) >= precisionIndex);
    return units.map(([int, unit]) => `${int}${unit[0]}`).join(" ") || `0${this.precision[0]}`;
  }
  const tense = this.tense;
  const micro = format === "micro";
  const inFuture = now.getTime() < date.getTime();
  const within = withinDuration(now, date, this.threshold);
  const locale = __classPrivateFieldGet(this, _RelativeTimeElement_instances, "a", _RelativeTimeElement_lang_get);
  if (typeof Intl !== "undefined" && Intl.RelativeTimeFormat) {
    const relativeFormat = new Intl.RelativeTimeFormat(locale, { numeric: "auto" });
    if (tense === "past" || tense === "auto" && !inFuture && within) {
      const [int, unit] = micro ? microTimeAgo(date) : timeAgo(date);
      if (micro)
        return `${int}${unit[0]}`;
      return relativeFormat.format(int, unit);
    }
    if (tense === "future" || tense === "auto" && inFuture && within) {
      const [int, unit] = micro ? microTimeUntil(date) : timeUntil(date);
      if (micro)
        return `${int}${unit[0]}`;
      return relativeFormat.format(int, unit);
    }
  }
  if (typeof Intl === "undefined" || !Intl.DateTimeFormat)
    return;
  const formatter = new Intl.DateTimeFormat(locale, {
    second: this.second,
    minute: this.minute,
    hour: this.hour,
    weekday: this.weekday,
    day: this.day,
    month: this.month,
    year: this.year,
    timeZoneName: this.timeZoneName
  });
  return `${this.prefix} ${formatter.format(date)}`.trim();
}, "_RelativeTimeElement_getFormattedDate");

// dist/relative-time-element-define.js
var root2 = typeof globalThis !== "undefined" ? globalThis : window;
try {
  customElements.define("relative-time", RelativeTimeElement);
  root2.RelativeTimeElement = RelativeTimeElement;
} catch (e) {
  if (!(root2.DOMException && e instanceof DOMException && e.name === "NotSupportedError") && !(e instanceof ReferenceError)) {
    throw e;
  }
}
var relative_time_element_define_default = RelativeTimeElement;

// dist/index.js
var dist_default = relative_time_element_define_default;
export {
  relative_time_element_define_default as RelativeTimeElement,
  RelativeTimeUpdatedEvent,
  dist_default as default
};
